/* painter.h */

#ifndef Y_BUFFER_PAINTER_H
#define Y_BUFFER_PAINTER_H

struct Painter;

#include <Y/y.h>
#include <Y/util/colour.h>
#include <Y/util/rectangle.h>

/* This defines an ABSTRACT Painter.
 *
 * Painters are very closely tied to buffers, i.e. a buffer might have
 * several painters associated with it, and painter implementations will
 * usually be defined within the file for a buffer implementation, since
 * it might need access to the internal structures of the buffer.
 */

void     painterDestroy (struct Painter *);

void     painterSaveState (struct Painter *);
void     painterRestoreState (struct Painter *);

void     painterTranslate (struct Painter *, double x, double y);
void     painterScale (struct Painter *, double x, double y);

void     painterClipTo (struct Painter *, struct Rectangle *);
void     painterEnter (struct Painter *, struct Rectangle *);

struct Rectangle *
         painterGetClipRectangle (struct Painter *);

int      painterFullyClipped (struct Painter *);

int      painterClipCoordinates (struct Painter *, int *x, int *y, int *w, int *h);

void     painterSetBlendMode (struct Painter *, enum ColourBlendMode);

void     painterSetPenColour (struct Painter *, uint32_t);
uint32_t painterGetPenColour (struct Painter *);
void     painterSetFillColour (struct Painter *, uint32_t);
uint32_t painterGetFillColour (struct Painter *);

/*
void  painterDrawText (struct Painter *, struct Font *, int x, int y,
                       const char *text);
*/

/* Virtual Methods *
 * =============== */

void   painterClearRectangle (struct Painter *, int x, int y, int w, int h);
void   painterDrawRectangle  (struct Painter *, int x, int y, int w, int h);
void   painterDrawHLine      (struct Painter *, int x, int y, int dx);
void   painterDrawVLine      (struct Painter *, int x, int y, int dy);
void   painterDrawLine       (struct Painter *, int x, int y, int dx, int dy);

/* renders a w by h filled rectangle at x,y in the current pen
   colour modulated by the alpha map given at alpha. */
void   painterDrawAlphamap (struct Painter *, unsigned char *alpha,
                            int x, int y, int w, int h, int s);
void   painterDrawRGBAData (struct Painter *, uint32_t *data,
                            int x, int y, int w, int h, int s);
/*
void   painterCopyPixmap (struct Painter *, struct Pixmap *, int x, int y);
*/
#endif /* Y_BUFFER_PAINTER_H */

/* arch-tag: a6488fab-d0e8-4027-80f3-6539dc3cda51
 */
