#include <Y/message/client.h>
#include <Y/message/client_p.h>
#include <Y/message/message.h>
#include <Y/util/index.h>

#include <Y/object/class.h>

#include <stdlib.h>
#include <unistd.h>

static struct Index *clients = NULL;
static int clientNextID = 1;

int
clientKeyFunction (void *key_v, void *obj_v)
{
  int *key_p = key_v;
  struct Client *obj = obj_v;
  if (*key_p == obj->id)
    return 0;
  if (*key_p < obj->id)
    return -1;
  else
    return 1;
}

int
clientComparisonFunction (void *obj1_v, void *obj2_v)
{
  struct Client *obj1 = obj1_v;
  struct Client *obj2 = obj2_v;
  if (obj1->id == obj2->id)
    return 0;
  if (obj1->id < obj2->id)
    return -1;
  else
    return 1;
}

void
clientRegister (struct Client *c)
{
  if (clients == NULL)
    {
      clients = indexCreate (clientKeyFunction, clientComparisonFunction);
      clientNextID = 1;
    }
  Y_TRACE ("Registering client with ID: %d", clientNextID);
  c -> id = clientNextID ++;
  c -> objects = indexCreate (objectKeyFunction, objectComparisonFunction);
  indexAdd (clients, c);
}

void
clientClose (struct Client *c)
{
  Y_TRACE ("Closing client %d", c->id);
  indexRemove (clients, &(c->id));
  indexDestroy (c -> objects, (void(*)(void*))objectDestroy);
  c -> c -> close (c);
}

int
clientGetID (struct Client *c)
{
  if (c == NULL)
    return 0;
  return c -> id;
}

struct Client *
clientFind (int id)
{
  if (clients == NULL)
    return NULL;
  return indexFind (clients, &id);
}

void
clientSendMessage (struct Client *c, struct Message *m)
{
  if (c != NULL)
    c -> c -> sendMessage (c, m);
}

void
clientAddObject (struct Client *c, struct Object *o)
{
  if (c == NULL)
    return;
  Y_TRACE ("Adding object %d [%s] to client %d",
          objectGetID (o), classGetName (objectClass (o)), c->id);
  indexAdd (c -> objects, o);
}

void
clientRemoveObject (struct Client *c, struct Object *o)
{
  int oid = objectGetID (o);
  indexRemove (c -> objects, &oid);
}


/* arch-tag: 8eb95cd6-369e-4879-961a-45aaef4d33c7
 */
