
#include <Y/y.h>
#include <Y/object/class_p.h>
#include <Y/object/object_p.h>
#include <Y/util/yutil.h>
#include <Y/util/index.h>
#include <string.h>

void yRegisterDefaultClasses ();

static struct Index *classNameIndex = NULL;
static struct Index *classIDIndex = NULL;
static int classNextID = 1;

static int
classNameKeyFunction (void *key_v, void *obj_v)
{
  char *key = key_v;
  struct Class *obj = obj_v;
  return strcmp (key, classGetName (obj));
}

static int
classNameComparisonFunction (void *obj1_v, void *obj2_v)
{
  struct Class *obj1 = obj1_v;
  struct Class *obj2 = obj2_v;
  return strcmp (classGetName (obj1), classGetName (obj2));
}

static int
classIDKeyFunction (void *key_v, void *obj_v)
{
  int *key_p = key_v;
  struct Class *obj = obj_v;
  int obj_id = classGetID (obj);
  if (*key_p == obj_id)
    return 0;
  return (*key_p < obj_id) ? -1 : 1;
}

static int
classIDComparisonFunction (void *obj1_v, void *obj2_v)
{
  struct Class *obj1 = obj1_v;
  int obj1_id = classGetID (obj1);
  struct Class *obj2 = obj2_v;
  int obj2_id = classGetID (obj2);
  if (obj1_id == obj2_id)
    return 0;
  return (obj1_id < obj2_id) ? -1 : 1;
}

void
classInitialise ()
{
  classNameIndex = indexCreate (classNameKeyFunction,
                                classNameComparisonFunction);
  classIDIndex = indexCreate (classIDKeyFunction, classIDComparisonFunction);
  yRegisterDefaultClasses ();
}

void
classFinalise ()
{
  indexDestroy (classNameIndex, NULL);
  indexDestroy (classIDIndex, NULL);
}

const char *
classGetName (struct Class *c)
{
  return c -> name;
}

int
classGetID (struct Class *c)
{
  return c -> id;
}

struct Object *
classInstantiate (struct Class *c, struct Client *client, struct Parameters *p)
{
  struct Object * o = c -> instantiate (p);
  clientAddObject (client, o);
  o -> client = client;
  return o;
}

void
classDespatch (struct Class *c, struct Object *o, struct Message *m)
{
  if (c -> despatch != NULL)
    c -> despatch (o, m);
}

int
classInherits (struct Class *c, const char *classname)
{
  struct Class *current = c;
  while (current != NULL)
    {
      if (strcmp (current -> name, classname) == 0)
        return 1;

      current = current -> super;
    }

  return 0;
}

static int
classCompareMethod (const void *key_v, const void *memb_v)
{
  const char *key = key_v;
  const struct Method *memb = memb_v;
  return strcmp (key, memb->name);
}

void
classInvokeMethod (struct Class *c, struct Object *o, const char *methodName,
                   struct Parameters *params)
{
  struct Method *method = bsearch (methodName, c->methods, c->methodCount,
                                   sizeof (struct Method), classCompareMethod);

  if (method != NULL)
    {
      method -> m (o, params);
    }
  else if (c -> super != NULL)
    {
      classInvokeMethod (c -> super, o, methodName, params);
    }
  else
    {
      parametersSetError (params);
      parametersAddOutput (params, "Method Not Found");
      parametersAddOutput (params, classGetName (c));
      parametersAddOutput (params, methodName); 
    }
}

void
classRegister (struct Class *c)
{
  c -> id = classNextID++;
  indexAdd (classNameIndex, c);
  indexAdd (classIDIndex, c);
}

struct Class *
classFindByName (const char *name)
{
  struct Class *c;
  if (classNameIndex == NULL)
    return NULL;
  c = indexFind (classNameIndex, (void *)name);
  return c;
}

struct Class *
classFindByID (int id)
{
  struct Class *c;
  if (classIDIndex == NULL)
    return NULL;
  c = indexFind (classIDIndex, &id);
  return c;
}


/* arch-tag: 8c8cb7e2-7d3d-4178-82cc-ab6db357bc36
 */
