
#include <Y/y.h>
#include <stdio.h>
#include <Y/util/yutil.h>
#include <Y/object/class_p.h>
#include <Y/object/object_p.h>
#include <Y/object/paramaters.h>

#define CHECK_STOP abort()
#include <Y/util/check.h>

char *checkName;
char *checkModule;

/* BEGIN UTILITY DEFINITIONS */

/* The testing will be done on a hierarchy of shapes;
 *            Shape
 *              |
 *     ________/_\_______
 *    |         |        |
 *  Square  Rectangle  Circle
 */

/*** SHAPE ***/

struct Class CShape =
{
  name: "Shape",
  super: NULL,
  methodCount: 0,
  methods: { }
};

/*** SQUARE ***/

void
squareArea (struct Object *o, struct Parameters *p)
{
  int sideLength = objectGetPropertyAsInteger(o, "side length");
  parametersAddOutputInteger (p, sideLength * sideLength);
}

void
squarePerim (struct Object *o, struct Parameters *p)
{
  int sideLength = objectGetPropertyAsInteger(o, "side length");
  parametersAddOutputInteger (p, sideLength * 4);
}

struct Class Square =
{
  name: "Square",
  super: Shape,
  methodCount: 2,
  methods: { { "area", squareArea }, { "perimeter", squarePerim } }
};

/*** RECTANGLE ***/

void
rectangleArea (struct Object *o, struct Parameters *p)
{
  int width = objectGetPropertyAsInteger(o, "width");
  int height = objectGetPropertyAsInteger(o, "height");
  parametersAddOutputInteger (p, width * height);
}

void
rectanglePerim (struct Object *o, struct Parameters *p)
{
  int width = objectGetPropertyAsInteger(o, "width");
  int height = objectGetPropertyAsInteger(o, "height");
  parametersAddOutputInteger (p, 2 * (width + height));
}

struct Class Rectangle =
{
  name: "Rectangle",
  super: Shape,
  methodCount: 2,
  methods: { { "area", rectangleArea }, { "perimeter", rectanglePerim } }
};

/*** CIRCLE ***/

void
circleArea (struct Object *o, struct Parameters *p)
{
  int radius = objectGetPropertyAsInteger(o, "radius");
  parametersAddOutputInteger (p, 3.1415926535 * radius * radius);
}

void
circlePerim (struct Object *o, struct Parameters *p)
{
  int radius = objectGetPropertyAsInteger(o, "radius");
  parametersAddOutputInteger (p, 2 * 3.1415926534 * radius);
}

struct Class Circle =
{
  name: "Circle",
  super: Shape,
  methodCount: 2,
  methods: { { "area", circleArea }, { "perimeter", circlePerim } }
};

/******/

int
object_check_methods ()
{
  struct Object *rect1, *square1, *circle1;

  checkModule = "methods";

  CHECK_THAT( ( rect1 = rectangleCreate (5, 10) ) != NULL );
  CHECK_THAT( ( square1 = squareCreate (8) ) != NULL );
  CHECK_THAT( ( circle1 = circleCreate (10) ) != NULL );

}

int
main (void)
{
  int failed = 0;
  checkName = "Object"; 
  failed = object_check_methods () ? 1 : failed;
  return failed;
}

/* arch-tag: 685fb0b1-307f-495e-90b0-7f9df16889db
 */
