#include <Y/util/rectangle.h>
#include <Y/util/yutil.h>

struct Rectangle *
rectangleCreate (int x, int y, int w, int h)
{
  struct Rectangle *rect = ymalloc (sizeof (struct Rectangle));
  rect -> x = x;
  rect -> y = y;
  rect -> w = w;
  rect -> h = h;
  return rect;
}

struct Rectangle *
rectangleDuplicate (const struct Rectangle *r)
{
  return rectangleCreate (r->x, r->y, r->w, r->h);
}

void
rectangleDestroy (struct Rectangle *self)
{
  yfree (self);
}

int
rectangleUnion (struct Rectangle *dest,
                const struct Rectangle *src1, const struct Rectangle *src2)
{
  int x0, y0, x1, y1, w, h, area0, area1;
  x0 = MIN (src1 -> x, src2 -> x);
  y0 = MIN (src1 -> y, src2 -> y);
  x1 = MAX (src1 -> x + src1 -> w, src2 -> x + src2 -> w);   
  y1 = MAX (src1 -> y + src1 -> h, src2 -> y + src2 -> h);
  w = x1 - x0;
  h = y1 - y0;
  area0 = src1 -> w * src1 -> h + src2 -> w * src2 -> h;
  area1 = w * h;
  if (dest)
    {
      dest -> x = x0;
      dest -> y = y0;
      dest -> w = w;
      dest -> h = h;
    }
  return area1 <= area0;
}

int
rectangleIntersect (struct Rectangle *dest,
                    const struct Rectangle *src1, const struct Rectangle *src2)
{
  int x, y, w, h;
  x = MAX (src1 -> x, src2 -> x);
  y = MAX (src1 -> y, src2 -> y);
  w = MIN (src1 -> x + src1 -> w, src2 -> x + src2 -> w) - x;
  h = MIN (src1 -> y + src1 -> h, src2 -> y + src2 -> h) - y;
  if (dest)
    {
      dest -> x = x;
      dest -> y = y;
      dest -> w = w;
      dest -> h = h;
    }
  return (w >= 0 && h >= 0);
}

void
rectanglelistUnionOverlaps (struct List *self)
{
  struct ListIterator *iter = listGetStartIterator (self);
  while (listiteratorHasValue (iter))
    {
      struct Rectangle *rect0 = listiteratorGet (iter);
      struct ListIterator *oter = listiteratorClone (iter);
      listiteratorNext (oter);
      while (listiteratorHasValue (oter))
        {
          struct Rectangle *rect1 = listiteratorGet (oter);
          if (rectangleIntersect (NULL, rect0, rect1))
            {
              rectangleUnion (rect0, rect0, rect1);
              listiteratorRemove (oter);
              yfree (rect1);
            }
          else
            listiteratorNext (oter);
        }
      listiteratorDestroy (oter);
      listiteratorNext (iter);
    }
  listiteratorDestroy (iter);
}

struct List *
rectanglelistIntersectWith (struct List *src1, struct List *src2)
{
  struct List *rc = listCreate ();
  struct ListIterator* iter1 = listGetStartIterator (src1);
  while (listiteratorHasValue (iter1))
    {
      struct ListIterator *iter2 = listGetStartIterator (src2);
      while (listiteratorHasValue (iter2))
        {
          struct Rectangle *intersection = ymalloc (sizeof (struct Rectangle));
          struct Rectangle *rect1 = listiteratorGet (iter1);
          struct Rectangle *rect2 = listiteratorGet (iter2);
          if (rectangleIntersect (intersection, rect1, rect2))
            {
              listAppend (rc, intersection);
            }
          else
            {
              yfree (intersection);
            }
          listiteratorNext (iter2);
        }
      listiteratorNext (iter1);
    }
  return rc;
}


/* arch-tag: 31e15c03-683d-4a2b-b028-ab7010389777
 */
