#include <Y/widget/label.h>
#include <Y/widget/widget_p.h>

#include <Y/util/yutil.h>
#include <Y/buffer/painter.h>

#include <Y/object/class_p.h>
#include <Y/object/object_p.h>
#include <Y/modules/theme.h>

#include <Y/text/font.h>

#include <stdio.h>
#include <ctype.h>
#include <string.h>

struct Label
{
  struct Widget widget;
};

static void labelReconfigure (struct Widget *);
static void labelPaint (struct Widget *, struct Painter *);

static struct Object *labelInstantiate (struct Parameters *);

static void labelTextSet (struct Property *, void *);

CLASSDEF struct Class labelClass =
{
  name:          "Label",
  super:         &widgetClass,
  instantiate:   labelInstantiate,
  destroy:       (void (*)(struct Object *))labelDestroy,
  methodCount:   0,
  methods:
    { {} }
};

static struct WidgetTable labelTable =
{
  c:             &labelClass,
  reconfigure:   labelReconfigure,
  paint:         labelPaint,
};

static inline struct Label *
castBack (struct Widget *widget)
{
  /* assert ( widget -> c == labelClass ); */
  return (struct Label *)widget;
}

static inline const struct Label *
castBackConst (const struct Widget *widget)
{
  /* assert ( widget -> c == labelClass ); */
  return (const struct Label *)widget;
}

static void
labelPaint (struct Widget *self_w, struct Painter *painter)
{
  int offset, width, ascender, descender;
  struct Font *font;
  const char *text =
               objectGetPropertyAsString (widgetToObject (self_w), "text");
  const char *alignment =
               objectGetPropertyAsString (widgetToObject (self_w), "alignment");

  int xp, yp;

  painterSetPenColour (painter, 0xFF000000);
  font = themeGetDefaultFont ();
  fontGetMetrics (font, &ascender, &descender, NULL); 
  fontMeasureString (font, text, &offset, &width, NULL);

  yp = (self_w -> h - ascender - descender ) / 2 + ascender;
  if (alignment && strcasecmp (alignment, "right") == 0)
    xp = (self_w -> w - width);
  else if (alignment && strcasecmp (alignment, "center") == 0)
    xp = (self_w -> w - width + offset) / 2 - offset;
  else
    xp = - offset;

  fontRenderString (font, painter, text, xp, yp);
}

struct Label *
labelCreate ()
{
  struct Label *self = ymalloc (sizeof (struct Label));
  objectInitialise (&(self -> widget.o), &labelClass);
  widgetInitialise (&(self -> widget), &labelTable);
  objectPropertyCallOnSet (labelToObject (self), "text", self, labelTextSet);
  objectPropertyCallOnSet (labelToObject (self), "alignment", self, labelTextSet);
  return self;
}

void
labelDestroy (struct Label *self)
{
  widgetFinalise (labelToWidget (self));
  objectFinalise (labelToObject (self));
  yfree (self);
}

struct Widget *
labelToWidget (struct Label *self)
{
  return &(self -> widget);
}

struct Object *
labelToObject (struct Label *self)
{
  return &(self -> widget.o);
}

static struct Object *
labelInstantiate (struct Parameters *p)
{
  return labelToObject (labelCreate ());
}

static void labelReconfigure (struct Widget *self_w)
{
  struct Label *self = castBack (self_w);
  struct Font *font;
  const char *text =
               objectGetPropertyAsString (widgetToObject (self_w), "text");
  int offset, width, ascender, descender;
  font = themeGetDefaultFont ();
  fontGetMetrics (font, &ascender, &descender, NULL);
  fontMeasureString (font, text, &offset, &width, NULL);

  self -> widget.minWidth = width - offset;
  self -> widget.minHeight = ascender + descender;
  self -> widget.reqWidth = width - offset;
  self -> widget.reqHeight = ascender + descender;

  widgetReconfigure (self -> widget.container);
}

static void
labelTextSet (struct Property *text, void *self_v)
{
  struct Label *self = self_v;
  widgetReconfigure (labelToWidget (self));
  widgetRepaint (labelToWidget (self), NULL);
}


/* arch-tag: 8badd23a-de08-4a6f-91b7-a99906b1a49d
 */
