
#include "libY.h"
#include "internal.h"

#include <stdlib.h>
#include <sys/select.h>
#include <unistd.h>
#include <stdio.h>

/* compile this file in if you want multi-fd access using select (); */

struct FDEntry
{
  int fd;
  int mask;
  void *data;
  void (*callback)(int, int, void*);
  struct FDEntry *next;
};

static struct FDEntry *entryList = NULL;

void
libY_initialise ()
{
  
}

void
libY_finalise ()
{

}

void
yRegisterFD (int fd, int mask, void *data, void (*call)(int, int, void*))
{
  /* check the fd is not already registered: */
  struct FDEntry *cur = entryList;
  while (cur != NULL)
    {
      if (cur -> fd == fd)
        return;
      cur = cur -> next;
    }

  cur = malloc (sizeof (struct FDEntry));
  if (cur == NULL)
    return;
  cur -> fd = fd;
  cur -> mask = mask;
  cur -> data = data;
  cur -> callback = call;
  cur -> next = entryList;
  entryList = cur;
}

void
yChangeFD (int fd, int mask)
{
  struct FDEntry *cur = entryList;
  while (cur != NULL)
    {
      if (cur -> fd == fd)
        {
          cur -> mask = mask;
          return;
        }
      cur = cur -> next;
    }
}

void
yUnregisterFD (int fd)
{
  struct FDEntry *prev = entryList;
  struct FDEntry *cur = entryList;
  if (cur == NULL)
    return;
  if (cur -> fd == fd)
    {
      entryList = cur -> next;
    }
  else
    {
      cur = cur -> next;
      while (cur != NULL)
        {
          if (cur -> fd == fd)
            {
              prev -> next = cur -> next;
              break;
            }
          prev = cur;
          cur = cur -> next;
        }
    }
  if (cur != NULL)
    free (cur);
}

void
libY_iterate (int timeout)
{
  int retval;
  int maxFD = 0;
  struct FDEntry *cur = entryList;
  struct timeval tv;
  fd_set read_fds, write_fds, except_fds;
  FD_ZERO (&read_fds);
  FD_ZERO (&write_fds);
  FD_ZERO (&except_fds);

  while (cur != NULL)
    {
      if (cur -> mask & Y_LISTEN_READ)
        FD_SET (cur -> fd, &read_fds);
      if (cur -> mask & Y_LISTEN_WRITE)
        FD_SET (cur -> fd, &write_fds);
      if (cur -> mask & Y_LISTEN_EXCEPT)
        FD_SET (cur -> fd, &except_fds);
      if (cur -> fd > maxFD)
        maxFD = cur -> fd;
      cur = cur -> next;
    }

  tv.tv_sec = 10;
  tv.tv_usec = 0;
  if (timeout >= 0)
    {
      tv.tv_sec = timeout / 1000;
      tv.tv_usec = (timeout % 1000) * 1000;
    }

  retval = select (maxFD + 1, &read_fds, &write_fds, &except_fds, &tv); 

  if (retval > 0)
    {
      cur = entryList;
      while (cur != NULL)
        {
          int mask = 0;
          if (FD_ISSET (cur -> fd, &read_fds))
            mask |= Y_LISTEN_READ;
          if (FD_ISSET (cur -> fd, &write_fds))
            mask |= Y_LISTEN_WRITE;
          if (FD_ISSET (cur -> fd, &except_fds))
            mask |= Y_LISTEN_EXCEPT;
          if (mask)
            {
              cur -> callback (cur -> fd, mask, cur -> data);
            }
          cur = cur -> next;
        }
    }
}


/* arch-tag: 227660d0-01dc-4e3c-8c92-24d83705bdf9
 */
