#include <Y/c++/object.h>
#include <Y/c++/y.h>
#include <Y/libY.h>

#include <iostream>
#include <sstream>

using std::ostringstream;
using std::pair;
using std::cout;
using std::endl;

map<int, Y::Object *> Y::Object::objects;

Y::Object::Object ()
{
}

Y::Object::~Object ()
{
  objects.erase (oid);
}

void
Y::Object::create (int classId)
{
  /* Creates the object on the server. */
  struct Message req, rep;

  req.to = 0;
  req.cid = classId;
  req.oid = 0;
  req.op = YMO_INSTANTIATE;
  req.meta = 0;
  req.ext = 0;
  req.data = NULL;
  req.len = 0;

  Y::sendMessage (&req);
  Y::recvMessage (&rep);

  oid = rep.oid;
  cid = classId;

  objects.insert (pair<int, Object *> (oid, this));
}

Y::Object *
Y::Object::find (int id)
{
  map<int, Object*>::iterator i = objects.find (id);
  if (i == objects.end() || (*i).first != id)
    return NULL;
  return (*i).second;
}

void
Y::Object::setProperty (const string& nameAndValue)
{
  /* Fundamental form of set property.
   * Takes the name and parameters in Y Message Format, i.e.
   * strings separated by ASCII Field Separator (0x1C) values.
   */
  struct Message req;
 
  req.to = 0;
  req.cid = cid;
  req.oid = oid;
  req.op = YMO_SET_PROPERTY;
  req.meta = 0;
  req.ext = 0;
  req.data = const_cast<char *>(nameAndValue.c_str());
  req.len = nameAndValue.length () + 1;

  Y::sendMessage (&req);
  /* no reply expected */
}

void
Y::Object::setProperty (const string &name, const string &value)
{
  ostringstream ss;
  ss << name << '\x1c' << value;
  setProperty (ss.str());
}

void
Y::Object::setProperty (const string &name, int value)
{
  ostringstream ss;
  ss << name << '\x1c' << value;
  setProperty (ss.str());
}

string
Y::Object::getProperty (const string &name)
{
  struct Message req, rep;

  req.to = 0;
  req.cid = cid;
  req.oid = oid;
  req.op = YMO_GET_PROPERTY;
  req.meta = 0;
  req.ext = 0;
  req.data = const_cast<char *>(name.c_str());
  req.len = name.length() + 1;

  Y::sendMessage (&req);
  Y::recvMessage (&rep);

  return rep.data;
}

void
Y::Object::subscribeSignal (const string &name)
{
  struct Message req;

  req.to = 0;
  req.cid = cid;
  req.oid = oid;
  req.op = YMO_SUBSCRIBE_SIGNAL;
  req.meta = 0;
  req.ext = 0;
  req.data = const_cast<char *>(name.c_str());
  req.len = name.length() + 1;

  Y::sendMessage (&req);
}

vector<string>
Y::Object::invokeMethod (const string &nameAndParams, bool expectReturn)
{
  struct Message req, rep;

  req.to = 0;
  req.cid = cid;
  req.oid = oid;
  req.op = YMO_INVOKE_METHOD;
  req.meta = 0;
  req.ext = 0;
  req.data = const_cast<char *>(nameAndParams.c_str());
  req.len = nameAndParams.length () + 1;

  if (expectReturn)
    req.meta |= 1<<0;

  Y::sendMessage (&req);

  if (expectReturn)
    {
      Y::recvMessage (&rep);
      return Y::parseParams (rep.data, rep.len);
    }
  else
    return vector<string> ();
}

void
Y::Object::onEvent (const string &name, const vector<string> &params)
{
  // Do nothing
}

/* arch-tag: fc2ef8c6-795a-484e-8b3a-73d63112d415
 */
